@@ -248,11 +248,11 @@ class Number(ConfigType):
 
     """Number class, base for Integer and Float.
 
+    :param num_type: the type of number used for casting (i.e int, float)
+    :param type_name: Type name to be used in the sample config file.
     :param min: Optional check that value is greater than or equal to min.
     :param max: Optional check that value is less than or equal to max.
-    :param type_name: Type name to be used in the sample config file.
     :param choices: Optional sequence of valid values.
-    :param num_type: the type of number used for casting (i.e int, float)
 
     .. versionadded:: 3.14
     """
@@ -358,9 +358,9 @@ class Float(Number):
 
     """Float type.
 
-    :param type_name: Type name to be used in the sample config file.
     :param min: Optional check that value is greater than or equal to min.
     :param max: Optional check that value is less than or equal to max.
+    :param type_name: Type name to be used in the sample config file.
 
     .. versionchanged:: 2.7
 
@@ -382,10 +382,10 @@ class Port(Integer):
 
     Represents a L4 Port.
 
-    :param type_name: Type name to be used in the sample config file.
-    :param choices: Optional sequence of valid values.
     :param min: Optional check that value is greater than or equal to min.
     :param max: Optional check that value is less than or equal to max.
+    :param type_name: Type name to be used in the sample config file.
+    :param choices: Optional sequence of valid values.
 
     .. versionadded:: 3.16
     """
@@ -440,23 +440,24 @@ class List(ConfigType):
         self.bounds = bounds
 
     def __call__(self, value):
-        if isinstance(value, list):
-            return value
+        if isinstance(value, (list, tuple)):
+            return list(value)
 
-        result = []
         s = value.strip()
-
         if self.bounds:
             if not s.startswith('['):
                 raise ValueError('Value should start with "["')
             if not s.endswith(']'):
                 raise ValueError('Value should end with "]"')
             s = s[1:-1]
+        if s:
+            values = s.split(',')
+        else:
+            values = []
+        if not values:
+            return []
 
-        if s == '':
-            return result
-
-        values = s.split(',')
+        result = []
         while values:
             value = values.pop(0)
             while True:
@@ -505,7 +506,7 @@ class Range(ConfigType):
     :param inclusive: True if the right bound is to be included in the range.
     :param type_name: Type name to be used in the sample config file.
 
-    .. versionadded:: 3.16
+    .. versionadded:: 3.18
     """
 
     def __init__(self, min=None, max=None, inclusive=True,
@@ -703,7 +704,7 @@ class IPAddress(ConfigType):
 
 
 class Hostname(ConfigType):
-    """Hostname type.
+    """Host domain name type.
 
     A hostname refers to a valid DNS or hostname. It must not be longer than
     253 characters, have a segment greater than 63 characters, nor start or
@@ -723,10 +724,14 @@ class Hostname(ConfigType):
         - Contains at least one character and a maximum of 63 characters
         - Consists only of allowed characters: letters (A-Z and a-z),
           digits (0-9), and hyphen (-)
+        - Ensures that the final segment (representing the top level domain
+          name) contains at least one non-numeric character
         - Does not begin or end with a hyphen
         - maximum total length of 253 characters
 
-        For more details , please see: http://tools.ietf.org/html/rfc1035
+        For more details , please see: http://tools.ietf.org/html/rfc1035,
+        https://www.ietf.org/rfc/rfc1912, and
+        https://tools.ietf.org/html/rfc1123
         """
 
         if len(value) == 0:
@@ -737,6 +742,10 @@ class Hostname(ConfigType):
         if value.endswith("."):
             value = value[:-1]
         allowed = re.compile("(?!-)[A-Z0-9-]{1,63}(?<!-)$", re.IGNORECASE)
+        if not re.search('[a-zA-Z-]', value.split(".")[-1]):
+            raise ValueError('%s contains no non-numeric characters in the '
+                             'top-level domain part of the host name and is '
+                             'invalid' % value)
         if any((not allowed.match(x)) for x in value.split(".")):
             raise ValueError("%s is an invalid hostname" % value)
         return value
@@ -751,6 +760,55 @@ class Hostname(ConfigType):
         return value
 
 
+class HostAddress(object):
+    """Host Address type.
+
+    Represents both valid IP addresses and valid host domain names
+    including fully qualified domain names.
+    Performs strict checks for both IP addresses and valid hostnames,
+    matching the opt values to the respective types as per RFC1912.
+
+    :param version: defines which version should be explicitly
+    checked (4 or 6) in case of an IP address
+    :param type_name: Type name to be used in the sample config file.
+
+    """
+
+    def __init__(self, version=None, type_name='host address value'):
+        """Check for valid version in case an IP address is provided
+
+        """
+
+        self.ip_address = IPAddress(version, type_name)
+        self.hostname = Hostname('localhost')
+
+    def __call__(self, value):
+        """Checks if is a valid IP/hostname.
+
+        If not a valid IP, makes sure it is not a mistyped IP before
+        performing checks for it as a hostname.
+
+        """
+
+        try:
+            value = self.ip_address(value)
+        except ValueError:
+            try:
+                value = self.hostname(value)
+            except ValueError:
+                raise ValueError("%s is not a valid host address", value)
+        return value
+
+    def __repr__(self):
+        return 'HostAddress'
+
+    def __eq__(self, other):
+        return self.__class__ == other.__class__
+
+    def _formatter(self, value):
+        return value
+
+
 class URI(ConfigType):
 
     """URI type
